package com.suribada.om;

import java.io.File;
import java.util.ArrayList;
import java.sql.*;
import com.suribada.ForumConstant;
import com.suribada.util.DBResource;
import com.suribada.exception.ForumDAOException;

public class OraForumDAOImpl implements ForumDAO {

	/**
	 * @see ForumDAO#countMessage(String)
	 */
	public int countMessage(String condition) throws ForumDAOException {
		int count = 0;
		DBResource db = new DBResource();
		Connection con = null;
		Statement stmt = null;
		ResultSet rs = null;
		String query = "SELECT COUNT(recno) FROM forum ";
		if (condition.length() > 0) {
			query = query + condition;
		}
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			if (rs.next()) {
				count = rs.getInt(1);
			}
		} catch(Exception e) {
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, stmt, con);
		}
		return count;
	}

	/**
	 * @see ForumDAO#delete(int, String)
	 */
	public void delete(int recno, String passwd, String path) throws ForumDAOException {
		DBResource db = new DBResource();
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		String query = "SELECT fid, thread, passwd, file_name FROM forum WHERE recno=?";
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, recno);
			rs = pstmt.executeQuery();
			rs.next();
			//  Էµ н尡  
			if (rs.getString("passwd") == null) {
				throw new ForumDAOException("You can't modify the message where password is null");
			}
			// н尡   
			if (!rs.getString("passwd").equals(passwd)) {
				throw new ForumDAOException("This Password dosn't match with the Origianl Password");				
			}
			int fid = rs.getInt("fid");
			String thread = rs.getString("thread");	
			String fileName = rs.getString("file_name");		
			query = "SELECT thread FROM forum WHERE fid=? AND LENGTH(thread)>? AND SUBSTR(thread, 0, ?)=? ";
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, fid);
			pstmt.setInt(2, thread.length());
			pstmt.setInt(3, thread.length());
			pstmt.setString(4, thread);
			rs = pstmt.executeQuery();
			// ڽ 尡    ʵ Ѵ.			
			if (rs.next()) {
				throw new ForumDAOException("Cannot delete the message which has replies");
			}	
			query = "DELETE FROM forum WHERE recno=?";
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, recno);
			pstmt.executeUpdate();
			if (fileName != null) {
				File file = new File(path + "/" + fileName);
				if (file.exists()) {
					file.delete();
				}
			}				
		} catch(Exception e)	{
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, pstmt, con);
		}
	}


	/**
	 * @see ForumDAO#getMessage(int)
	 */
	public Message getMessage(int recno) throws ForumDAOException {
		String query = "SELECT fid, thread, name, email, title, content FROM forum WHERE recno=?";
		DBResource db = new DBResource();
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Message message = null;
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, recno);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				message = new Message();
				message.setFid(rs.getInt("fid"));
				message.setThread(rs.getString("thread"));
				message.setName(rs.getString("name"));
				message.setEmail(rs.getString("email"));
				message.setTitle(rs.getString("title"));
				message.setContent(rs.getString("content"));
			}
		} catch(Exception e)	{
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, pstmt, con);
		}
		return message; 			
	}

	/**
	 * @see ForumDAO#getMessageList(int, int, int, String)
	 */
	public ArrayList getMessageList(int cpage, int range, int totalCnt, String condition)
		throws ForumDAOException 
	{
  		if (totalCnt == 0) {
	      	return null;
  		}
   		int first = range * (cpage-1);
   		String query = "SELECT * FROM ( SELECT /*+ index_desc(forum idx_forum_fid) */ ROWNUM rseq, recno, fid, thread, name, title,"
   					+ " TO_CHAR(sign_date, 'yyyy-mm-dd') sign_date, REF FROM forum ";
		if (condition.length() > 0) {
			query = query + condition + " AND ";
		} else {
			query = query + " WHERE ";
		}
		query = query + " ROWNUM<=" + (first + range) + ") WHERE rseq>" + first;
		System.out.println("query: " + query);
		ArrayList list = new ArrayList();
		DBResource db = new DBResource();
		Connection con = null;
		Statement stmt = null;
		ResultSet rs = null;
		Message message = null;
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			while (rs.next()) {
				message = new Message();
				message.setRecno(rs.getInt("recno"));
				message.setFid(rs.getInt("fid"));
				message.setThread(rs.getString("thread"));
				message.setName(rs.getString("name"));
				message.setTitle(rs.getString("title"));
				message.setSignDate(rs.getString("sign_date"));
				message.setRef(rs.getInt("ref"));
				list.add(message);
			}
		} catch(Exception e)	{
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, stmt, con);
		}
		System.out.println("list: " + list.size());
		return list; 			
	}

	/**
	 * @see ForumDAO#getMessageGroup(int)
	 */
	public ArrayList getMessageGroup(int recno, int fid) throws ForumDAOException {
		String query = "SELECT recno, thread, name, email, title, content, "
						+ " TO_CHAR(sign_date, 'yyyy-mm-dd hh24:mi:ss') sign_date, file_name, file_size " 
						+ " FROM forum WHERE fid=? ORDER BY thread ";
		ArrayList list = new ArrayList();
		DBResource db = new DBResource();
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Message message = null;
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, fid);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				message = new Message();
				message.setRecno(rs.getInt("recno"));
				message.setThread(rs.getString("thread"));
				message.setName(rs.getString("name"));
				message.setEmail(rs.getString("email"));
				message.setTitle(rs.getString("title"));
				message.setContent(rs.getString("content"));
				message.setSignDate(rs.getString("sign_date"));
				message.setFileName(rs.getString("file_name"));
				message.setFileSize(rs.getInt("file_size"));
				list.add(message);
			}
			query = "UPDATE forum SET ref=ref+1 WHERE recno=?";
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, recno);
			pstmt.executeUpdate();
		} catch(Exception e)	{
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, pstmt, con);
		}
		System.out.println("list: " + list.size());
		return list; 					
	}

	/**
	 * @see ForumDAO#insert(Message)
	 */
	public void insert(Message message) throws ForumDAOException {
		DBResource db = new DBResource();
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		String queryRecno = "SELECT forum_seq.NEXTVAL FROM dual";  
		String queryThread = "SELECT thread FROM (SELECT thread FROM forum WHERE fid=? AND LENGTH(thread)=? AND SUBSTR(thread, 0, ?)=? ORDER BY thread DESC) WHERE rownum=1";
		String queryFid = "SELECT NVL(MAX(fid), 0)+1 FROM forum";
		String query = "INSERT INTO forum VALUES(?, ?, ?, ?, ?, ?, ?, ?, SYSDATE, 0, ?, ?)";
		int rowChanged = 0; 
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			pstmt = con.prepareStatement(queryRecno);
			rs = pstmt.executeQuery();
			rs.next();
			int recno = rs.getInt(1);
			String thread = message.getThread();
			int fid = message.getFid();
			if (fid > 0) { // 亯
				pstmt = con.prepareStatement(queryThread);
				pstmt.setInt(1, message.getFid());
				pstmt.setInt(2, thread.length()+1);
				pstmt.setInt(3, thread.length());
				pstmt.setString(4, thread);
				rs = pstmt.executeQuery();
				if (rs.next()) {
					char last = (char) (rs.getString(1).charAt(thread.length()) + 1);
					thread = thread + last;
				}
				else {
					thread = thread + "A";
				}
			} else { 
				pstmt = con.prepareStatement(queryFid);
				rs = pstmt.executeQuery();
				rs.next();
				fid = rs.getInt(1);	
			}			
			pstmt = con.prepareStatement(query);
			
			pstmt.setInt(1, recno);
			pstmt.setInt(2, fid);
			pstmt.setString(3, thread);
			pstmt.setString(4, message.getName());
			pstmt.setString(5, message.getEmail());
			pstmt.setString(6, message.getTitle());
			pstmt.setString(7, message.getContent());
			pstmt.setString(8, message.getPasswd());
			pstmt.setString(9, message.getFileName());
			pstmt.setInt(10, message.getFileSize());

			pstmt.executeUpdate();
		} catch(Exception e) {
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, pstmt, con);
		}
	}

	/**
	 * @see ForumDAO#update(Message)
	 */
	public void update(Message message) throws ForumDAOException {
		DBResource db = new DBResource();
		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		String query = "SELECT passwd FROM forum WHERE recno=?";  
		try {
			con = db.getConnection(ForumConstant.FORUM_DATASOURCE);
			pstmt = con.prepareStatement(query);
			pstmt.setInt(1, message.getRecno());
			rs = pstmt.executeQuery();
			rs.next();
			if (rs.getString(1) == null) {
				throw new ForumDAOException("You can't modify the message where password is null");
			}
			if (!rs.getString(1).equals(message.getPasswd())) {
				throw new ForumDAOException("This Password dosn't match with the Origianl Password");				
			}
			query = "UPDATE forum SET email=?, title=?, content=?, file_name=?, file_size=? WHERE recno=?";
			pstmt = con.prepareStatement(query);
			pstmt.setString(1, message.getEmail());
			pstmt.setString(2, message.getTitle());
			pstmt.setString(3, message.getContent());
			pstmt.setString(4, message.getFileName());
			pstmt.setInt(5, message.getFileSize());
			pstmt.setInt(6, message.getRecno());
			pstmt.executeUpdate();
		} catch(Exception e) {
			throw new ForumDAOException(e.getMessage());
		} finally {
			db.close(rs, pstmt, con);
		}
	
	}

}
